--[[

		Sonic Adventure Style's Checkpoint

Contributors: Ace Lite
@Team Blue Spring 2022-2024

]]

local MapthingCheckpoints = {thg = {}; dis = {};}

local Disable_Checkpoints = false

addHook("MapLoad", function()
	Disable_Checkpoints = false
	if CV_FindVar("dc_checkpoints").value == 0 then
		Disable_Checkpoints = true
	end
end)

addHook("MapChange", function()
	MapthingCheckpoints.thg = {}
	MapthingCheckpoints.dis = {}
end)

addHook("MobjSpawn", function(a, mt)
	if Disable_Checkpoints then return end

		-- Set up of Adventure Checkpoint
		a.state = S_INVISIBLE
		a.sprite = SPR_CHE0
		a.frame = A
		a.flags2 = $|MF2_DONTDRAW
		a.base = {}
		a.pads = {}
		a.stick = {}
		a.bulb = {}

		-- Model build

		table.insert(MapthingCheckpoints.thg, a)
		local thingnum = MapthingCheckpoints.thg
		a.idmt = #thingnum

		if not MapthingCheckpoints.dis[a.idmt] then
			MapthingCheckpoints.dis[a.idmt] = {}
			MapthingCheckpoints.dis[a.idmt].dis = 1280
		end

		for i = 1,2 do
			local ang = a.angle+ANGLE_180*i-ANGLE_90

			local base = P_SpawnMobjFromMobj(a, 73*cos(ang), 73*sin(ang),0, MT_FRONTERADUMMY)
			base.state = S_INVISIBLE
			base.sprite = a.sprite
			base.angle = ang
			base.frame = A
			base.flags = $|MF_NOGRAVITY|MF_NOCLIP|MF_NOCLIPHEIGHT
			base.flags2 = $|MF2_LINKDRAW

			local pad = P_SpawnMobjFromMobj(a, 56*cos(ang), 56*sin(ang),0, MT_BACKTIERADUMMY)
			pad.state = S_INVISIBLE
			pad.sprite = a.sprite
			pad.angle = ang
			pad.frame = B
			pad.flags = $|MF_NOGRAVITY|MF_NOCLIP|MF_NOCLIPHEIGHT
			pad.flags2 = $|MF2_LINKDRAW

			local stick = P_SpawnMobjFromMobj(base, 0, 0, 44*FRACUNIT*(a.flags2 & MF2_OBJECTFLIP and -1 or 1), MT_BACKTIERADUMMY)
			stick.state = S_INVISIBLE
			stick.sprite = a.sprite
			stick.angle = ang
			if i == 1 then
				stick.frame = C|FF_PAPERSPRITE
				stick.rollangle = ANGLE_180
				stick.angle = ang+ANGLE_180
			else
				stick.frame = C|FF_PAPERSPRITE
			end
			stick.flags = $|MF_NOGRAVITY|MF_NOCLIP|MF_NOCLIPHEIGHT
			stick.flags2 = $|MF2_LINKDRAW

			local bulb = P_SpawnMobjFromMobj(base, -49*cos(ang), -49*sin(ang),36*FRACUNIT*(a.flags2 & MF2_OBJECTFLIP and -1 or 1), MT_FRONTERADUMMY)
			bulb.state = S_INVISIBLE
			bulb.sprite = a.sprite
			bulb.angle = ang
			bulb.frame = E
			bulb.flags = $|MF_NOGRAVITY|MF_NOCLIP|MF_NOCLIPHEIGHT
			bulb.flags2 = $|MF2_LINKDRAW

			table.insert(a.base, base)
			table.insert(a.pads, pad)
			table.insert(a.stick, stick)
			table.insert(a.bulb, bulb)

		end

end, MT_STARPOST)

sfxinfo[freeslot("sfx_advche")].caption = "Checkpoint"
mobjinfo[MT_STARPOST].painsound = sfx_advche

addHook("MobjThinker", function(a)
	if Disable_Checkpoints then return end

	local thglist = MapthingCheckpoints.thg
	if a.checksurrondings == nil and #thglist > 1 then
		for k,rvmt in ipairs(MapthingCheckpoints.thg) do
			if rvmt and rvmt.valid then
				local dist = P_AproxDistance(a.x - rvmt.x, a.y - rvmt.y)/FRACUNIT
				local distz = abs(a.z - rvmt.z)
				--print(distz)

				if distz < 50*FRACUNIT and dist < MapthingCheckpoints.dis[a.idmt].dis and a.health == rvmt.health and k ~= a.idmt then
					MapthingCheckpoints.dis[a.idmt].x = rvmt.x
					MapthingCheckpoints.dis[a.idmt].y = rvmt.y
					MapthingCheckpoints.dis[a.idmt].dis = dist
					--print("yes, checkpoints are triggered")
					a.ohnomorecheckpoints = 1
				end
			end
		end
		--print("yes, checked surrondings")
		a.checksurrondings = 1
	end

	local ang = a.angle-ANGLE_90
	if a.ohnomorecheckpoints ~= nil then
		ang = R_PointToAngle2(a.x, a.y, MapthingCheckpoints.dis[a.idmt].x or 0, MapthingCheckpoints.dis[a.idmt].y or 0)
	end

	if a.ohnomorecheckpoints ~= nil and a.pads[2] then
		P_TryMove(a, a.x+25*cos(ang), a.y+25*sin(ang), true)
		P_RemoveMobj(a.base[2])
		P_RemoveMobj(a.stick[2])
		P_RemoveMobj(a.bulb[2])
		P_RemoveMobj(a.pads[2])
		--print("yes, checkpoints are cleaned")
		a.pads[2], a.base[2], a.stick[2], a.bulb[2] = nil,nil,nil,nil
	end

	for id,pad in ipairs(a.pads) do
		pad.angle = ang+ANGLE_180*id
		P_TeleportMove(pad,
		a.x+56*cos(ang+ANGLE_180*id),
		a.y+56*sin(ang+ANGLE_180*id),
		a.z)
	end

	for id,base in ipairs(a.base) do
		base.angle = ang+ANGLE_180*id
		P_TeleportMove(base,
		a.x+73*cos(ang+ANGLE_180*id),
		a.y+73*sin(ang+ANGLE_180*id),
		a.z)
	end

	if a.state == S_STARPOST_SPIN then

		local decreasespd = ease.linear((a.tics*FRACUNIT)/states[S_STARPOST_SPIN].tics, 47*ANG1, 36*ANG1)

		for id,stick in ipairs(a.stick) do
			stick.angle =  $ + (id == 1 and decreasespd or -decreasespd)
			P_TeleportMove(stick, a.base[id].x, a.base[id].y, a.z+44*FRACUNIT*(a.flags2 & MF2_OBJECTFLIP and -1 or 1))
		end

		for id,bulb in ipairs(a.bulb) do
			bulb.frame = F
			bulb.angle =  $ + (id == 1 and decreasespd or -decreasespd)
			P_TeleportMove(bulb, a.base[id].x-49*cos(bulb.angle), a.base[id].y-49*sin(bulb.angle), a.z+36*FRACUNIT*(a.flags2 & MF2_OBJECTFLIP and -1 or 1))
		end
	elseif a.state == S_STARPOST_FLASH then
		if a.angv == nil then
			a.angv = 0
		end

		if a.angv <= 105 then
			a.angv = $+5
		end

		local calcangle = ease.outquad((a.angv*FRACUNIT)/110, 0, -90*ANG1)
		local decreasespd = ease.outquint((a.angv*FRACUNIT)/110, 36*ANG1, 0)
		local height = ease.outquad((a.angv*FRACUNIT)/110, 0, 58)

		if a.angv < 110 then

			for id,stick in ipairs(a.stick) do
				stick.rollangle = (id == 1 and ANGLE_180-calcangle or calcangle)
				stick.angle =  $ + (id == 1 and decreasespd or -decreasespd)

				P_TeleportMove(stick,
				a.base[id].x,
				a.base[id].y,
				a.z+38*FRACUNIT*(a.flags2 & MF2_OBJECTFLIP and -1 or 1))
			end

			for id,bulb in ipairs(a.bulb) do
				bulb.angle =  $ + (id == 1 and decreasespd or -decreasespd)

				P_TeleportMove(bulb,
				a.base[id].x-57*cos(bulb.angle)+height*cos(bulb.angle),
				a.base[id].y-57*sin(bulb.angle)+height*sin(bulb.angle),
				a.z+(34+height)*FRACUNIT*(a.flags2 & MF2_OBJECTFLIP and -1 or 1))
			end
		end

		if a.angv == 110 then
			if a.stick[1].frame ~= G or (a.stick[2] and a.stick[2].frame ~= G) then
				for id,stick in ipairs(a.stick) do
					stick.rollangle = 0
					stick.frame = G
				end
			end

			for id,bulb in ipairs(a.bulb) do
				bulb.frame = F
				P_TeleportMove(bulb,
				a.base[id].x,
				a.base[id].y,
				a.z+(33+height)*FRACUNIT*(a.flags2 & MF2_OBJECTFLIP and -1 or 1))
			end

			for id,stick in ipairs(a.stick) do
				P_TeleportMove(stick,
				a.base[id].x,
				a.base[id].y,
				a.z+34*FRACUNIT*(a.flags2 & MF2_OBJECTFLIP and -1 or 1))
			end

		end

	else

		for id,stick in ipairs(a.stick) do
			stick.angle = ang
			P_TeleportMove(stick, a.base[id].x, a.base[id].y, a.z+44*FRACUNIT*(a.flags2 & MF2_OBJECTFLIP and -1 or 1))
		end

		for id,bulb in ipairs(a.bulb) do
			bulb.angle = ang+ANGLE_180*id
			P_TeleportMove(bulb, a.base[id].x-49*cos(bulb.angle), a.base[id].y-49*sin(bulb.angle), a.z+36*FRACUNIT*(a.flags2 & MF2_OBJECTFLIP and -1 or 1))
		end

	end

end,  MT_STARPOST)

addHook("PlayerThink", function(p)
	if Disable_Checkpoints then return end

	if not p.boxdisplay or not p.boxdisplay.timer then
		p.boxdisplay = {}
	end

	if p.boxdisplay.timer then
		p.boxdisplay.timer = $ - 1
	end

	if p.checkpointtime then
		p.checkpointtime = $ - 1
	end
end)

